               job  Split long tape records and recombine them
               ctl  6611
     *
     * Split long tape records into short ones.
     * Write a record before each short one to facilitate recombining
     *
     * Col 1.   0 - do not write previous record
     *          1 - write previous record
     *          2 - write previous record and endfile
     * Col 2-4. Address to read next record
     * Col 5-7. Address of GMWM if Col1. is 1 or 2.
     * Col 8-80 not used
     *
     * Halts (A- and B-addresses):
     *
     * 111 Recombiner finished normally, push start for another file
     * 222 More than ten read errors, push start to retry
     * 333 More than ten write errors, push start to skip and retry
     * 666 Read error for boot record of recombiner
     * 777 Splitter finished normally, push start for another file
     *
     * Sense Switches
     * B skip noise test
     *
     size      equ  1000  Maximum size of split blocks
     *
     x1        equ  89
     x2        equ  94
     x3        equ  99
     *
               org  81
     gm        dc   @}@
     *
     in        equ  6  tape with long records to be read
     inout     equ  1  tape with short records
     out       equ  3  tape on which to write long records
     kin       dc   6
     kinout    dc   1
     kout      dc   3
     *
     * Read the indicator record
     *
               org  100
     recomb    sw   gm
               b    clear
               mn   kinout,rdtp&3   Set to read from inout
               mn   kout,wrtp&3     Set to write on out
     rdind     sbr  x2,1
               b    tprd
               bce  nxread,1,0   Read next piece of the record
               mcw  7,x1         Set where to put GMWM
               lca  gm,0&x1      Set GMWM
               sbr  wrtp&6,buf   Set write address
               b    tpwr         go write it.
               cw   0&x1         Clear WM at end of record
               bce  nxread,1,1   Not at end of file
               wtm  out
               wtm  out
               bsp  out
               nop  111
               h
               b    rdind        Do another file
     *
     * Read the data record
     *
     nxread    mcw  4,x2
               b    tprd
               b    rdind
     *
     * Tape read routine
     *
     tprd      sbr  tprx&3
               mz   clearl,aztest&7  Set A-sone in test instruction
               mcw  aztest&7,12&x2  Set A-zone for noise test
     rdtpz     sbr  erct,0
               sw   eofin          Set EOF flag
     rdtp      rtw  in,0&x2
               sbr  x3             Addr of first character after record
               ber  rderr          Q. Error
               bef  tprx           Q. EOF, do not test for noise
               cw   eofin          Clear EOF switch
               bss  tprx,b         Q. Skip noise test
     aztest    bce  rdtp,12&x2,    Changed to A-zone for noise test
     tprx      b    0-0
     rderr     a    *-6,erct       Bump error count
               mn   rdtp&3,*&4     Get unit number
               bsp  0-0
               bce  rdtp,erct-1,0  Branch if fewer than ten retries
               nop  222
               h
               b    rdtpz
     *
     * Tape write routine
     *
     tpwr      sbr  tpwrx&3
     wrtpz     sbr  erct,0
     wrtp      wtw  out,0-0
               sbr  endwr          Addr of first character after record
               ber  wterr          Q. Error
     tpwrx     b    0-0
     wterr     a    *-6,erct       Bump error count
               mn   wrtp&3,*&4     Get unit number
               bsp  0-0
               bce  wrtp,erct-1,0  Branch if fewer than ten retries
               skp  0              Set skip latch for next write
               nop  333
               h
               b    wrtpz
     endwr     dcw  #3
     erct      dcw  #3
     eofin     dc   #1             Word mark if EOF on input
     *
     * Clear core down to buf
     *
     clear     sbr  clearx&3
               sw   buf
               sbr  clearl&3,0
     clearl    cs   0-0
               sbr  clearl&3
               bw   clearl,buf
     clearx    bin  0-0,
     *
               dcw  @}@            gmwm to stop writing the recombiner
               job  split long records into short ones
     *
     * Error check and link routine for starting recombiner.
     * Loads at 1-27.
     *
               ber  13             Q. Error
               cs   recomb,80      No, clear 0-80 and branch
               bsp  1
               nop  666
               h
               bin  18,
     endld     equ  *
     *
     * Write the recombiner
     *
     start     mz   clearl,aztest&7  Set A-zone to test for noise record
               lca  endld,27
               chain5
               mn   kin,rdtp&3     Set to read from in
               sbr  x2,buf         Set to read into buf
               mn   kinout,wrtp&3  Set to write on inout
               sbr  wrtp&6,1       Set the write address
               b    tpwr           Write the recombiner
               cs   80
               sw   2,5
               sw   gm             Now set a WM at the GM at 81
     anothr    lca  @0@,1
     *
     * Read a record
     *
     rdlp      b    clear          Clear top of core down to buf
               b    tprd           Read a recprd
               bw   endin,eofin    Q. EOF
               sbr  4,buf          Set BUF address in indicator record
               sbr  wrtp&6,1       Set write address to 1
               b    tpwr           Write the indicator record
               sw   15999&x3       Put a WM under the GM from the read
               sbr  7,15999&x3     Set address of GMWM in indicator
     *
     * Write it 1k at a time
     *
               sbr  x1,buf         Start writing at BUF
     wrlp      mcw  size&x1,svch   Save character where GMWM will be
               sw   svwm           Assume it has a WM
               bw   *&5,size&x1    Q. Does it have a WM
               cw   svwm           No
               lca  gm,size&x1     Put GMWM at next SIZE characters
               sbr  wrtp&6,0&x1    Set write address
               b    tpwr           Write a piece of the record
               c    x3,endwr       Q. At end of input record
               bu   notend         no
               sbr  sx1,0&x1
               sbr  sx3,0&x3
               mn   @00@,sx3
               mn
               mn   @00@,sx1
               mn
               c    sx1,sx3        Q. End and start in same hundreds
               bu   long           No
               bsp  inout          Backspace over short record
               cw   15999&x3       Clear GMWM
               lca  gm,100&x1      Set a GMWM for 100 chars
               b    tpwr
     long      lca  @1@,1
               b    rdlp
     notend    lca  @0@,1
               sbr  4,size&x1      Set address of chunk in indicator
               sbr  wrtp&6,1       Set write address to 1
               b    tpwr           Write the next piece indicator
               lca  svch,size&x1   Put the saved character back
               bw   *&5,svwm       Q. Did it have a WM
               cw   size&x1        No
               sbr  x1,size&x1     Bump the chunk address
               b    wrlp           Go write the next piece
     *
     endin     lca  @2@,1          Set end of file signal in indicator
               sbr  wrtp&6,1
               b    tpwr           Write the EOF indicator record
               wtm  inout          Write tape
               wtm  inout            marks and
               bsp  inout              backspace
               bsp  inout                over them
               nop  777
               h
               b    anothr         Process another file
     *
     endrd     dcw  #3
     svch      dcw  #1
     svwm      dcw  #1
     sx1       dcw  #3             Work area for X1 fiddling
     sx3       dcw  #3             Work area for X3 fiddling
               ltorg*
               org  *&x00
     buf       dc   #1
               end  start
